@@ -26,8 +26,11 @@
  *-----------------------------------------------------------------------------
  *
  * History:
- *	$Id: yeti.h,v 1.1 2006/07/19 08:50:02 eric Exp $
+ *	$Id: yeti.h,v 1.2 2007/12/26 10:56:39 eric Exp $
  *	$Log: yeti.h,v $
+ *	Revision 1.2  2007/12/26 10:56:39  eric
+ *	 - Update for management of dimension lists.
+ *
  *	Revision 1.1  2006/07/19 08:50:02  eric
  *	Initial revision
  *
@@ -266,33 +269,26 @@ PLUG_API Dimension *tmpDims;
 	garbage there when you are done for the next guy to clean up --
 	your part of the perpetual cleanup comes first. */
 
-extern void yeti_reset_dims(void);
-/*----- Prepare global variable tmpDims for the building of a new dimension
-	list (i.e. takes care of freeing old dimension list). */
+extern void yeti_reset_dimlist(void);
+/*----- Prepares global variable tmpDims for the building of a new
+	dimension list (i.e. takes care of freeing old dimension
+	list if any). */
 
-#define yeti_append_dimension(number, origin) \
-	(tmpDims = NewDimension(number, origin, tmpDims))
-/*----- Append a new temporary dimension (i.e. insert dimension before
-	tmpDims) and return tmpDims. */
+extern Dimension *yeti_grow_dimlist(long number);
+/*----- Appends dimension of length NUMBER to tmpDims and returns new
+	value of tmpDims.  Note that tmpDims must have been properly
+	initialized by yeti_reset_dimlist or yeti_start_dimlist (which
+	to see).  For instance to build the dimension list of a NCOLS
+	by NROWS array, do:
+	  yeti_reset_dimlist();
+	  yeti_grow_dimlist(ncols);
+	  yeti_grow_dimlist(nrows); */
 
-extern Dimension *yeti_first_dimension(long number, long origin);
-/*----- Initialize global temporary tmpDims with a single first dimension
-	with length NUMBER and starting index ORIGIN (usually 1).  The
-	routine takes care to free and update tmpDims (i.e. no needs to
-	call ResetDims before) and returns the new value of tmpDims.  E.g.,
-	to build a dimension list for a vector:
-	    tmpDims= yeti_first_dimension(number1, origin1);
-	or just:
-	    yeti_first_dimension(number1, origin1);
-	since yeti_first_dimension stores its result in tmpDims;  for a
-        matrix:
-	    yeti_first_dimension(number1, origin1);
-            yeti_append_dimension(number2, origin2);
-	or alternatively:
-	    yeti_reset_dims();
-	    yeti_append_dimension(number1, origin1);
-	    yeti_append_dimension(number2, origin2);
-	*/
+extern Dimension *yeti_start_dimlist(long number);
+/*----- Initializes global temporary tmpDims with a single first
+	dimension with length NUMBER and returns tmpDims.  Same as:
+	   yeti_reset_dimlist();
+	   yeti_grow_dimlist(number); */
 
 extern Dimension *yeti_make_dims(const long number[], const long origin[],
 				 size_t ndims);
