@@ -26,8 +26,16 @@
  *-----------------------------------------------------------------------------
  *
  * History:
- *	$Id: yeti_hash.c,v 1.7 2007/03/23 11:53:53 eric Exp eric $
+ *	$Id: yeti_hash.c,v 1.9 2007/12/26 10:56:21 eric Exp $
  *	$Log: yeti_hash.c,v $
+ *	Revision 1.9  2007/12/26 10:56:21  eric
+ *	 - Update for management of dimension lists.
+ *
+ *	Revision 1.8  2007/07/16 09:23:52  eric
+ *	 - Code cleanup.
+ *	 - Fix inconsistent "critical code begin/end" comment (thancks
+ *	   to Jean Pichon).
+ *
  *	Revision 1.7  2007/03/23 11:53:53  eric
  *	 - Hash table object can now have their own evaluator, which can be
  *	   queried/set by the `h_evaluator` function.
@@ -573,7 +581,7 @@ void Y_h_pop(int nargs)
       if (H_MATCH(entry, key, sname, len)) {
 	/* Delete the entry: (1) remove entry from chained list of entries in
 	   its slot, (2) pop contents of entry, (3) free entry memory. */
-	/* CRITICAL SECTION BEGIN */
+	/* CRITICAL CODE BEGIN */
 	if (prev) prev->next = entry->next;
 	else table->slot[index] = entry->next;
 	stack->ops   = entry->sym_ops;
@@ -581,7 +589,7 @@ void Y_h_pop(int nargs)
 	h_free(entry);
 	--table->number;
 	sp = stack; /* sp updated AFTER new stack element finalized */
-	/* CRITICAL SECTION END */
+	/* CRITICAL CODE END */
 	return; /* entry found and popped */
       }
       prev = entry;
@@ -621,7 +629,7 @@ void Y_h_keys(int nargs)
   table = get_hash(sp);
   number = table->number;
   if (number) {
-    result = YETI_PUSH_NEW_Q(yeti_first_dimension(number, 1));
+    result = YETI_PUSH_NEW_Q(yeti_start_dimlist(number));
     j = 0;
     for (i=0 ; i<table->size ; ++i) {
       for (entry=table->slot[i] ; entry ; entry=entry->next) {
@@ -720,7 +728,7 @@ void Y_h_stat(int nargs)
   table = get_hash(sp);
   number = table->number;
   slot = table->slot;
-  array = YETI_PUSH_NEW_ARRAY_L(yeti_first_dimension(number + 1, 1));
+  array = YETI_PUSH_NEW_ARRAY_L(yeti_start_dimlist(number + 1));
   result = array->value.l;
   for (i=0 ; i<=number ; ++i) result[i] = 0;
   for (i=0 ; i<table->size ; ++i) {
@@ -1084,7 +1092,7 @@ int h_remove(h_table_t *table, const char *sname)
       /* Delete the entry: (1) remove entry from chained list of entries in
          its slot, (2) unreference contents of entry, (3) free entry
          memory. */
-      /* CRITICAL SECTION BEGIN */
+      /* CRITICAL CODE BEGIN */
       if (prev) prev->next = entry->next;
       else table->slot[index] = entry->next;
       if (entry->sym_ops == &dataBlockSym) {
@@ -1093,7 +1101,7 @@ int h_remove(h_table_t *table, const char *sname)
       }
       h_free(entry);
       --table->number;
-      /* CRITICAL SECTION END */
+      /* CRITICAL CODE END */
       return 1; /* entry found and deleted */
     }
     prev = entry;
@@ -1154,7 +1162,7 @@ int h_insert(h_table_t *table, const char *sname, Symbol *sym)
   /* Replace contents of the entry with same key name if it already exists. */
   for (entry=table->slot[key&table->mask] ; entry ; entry = entry->next) {
     if (H_MATCH(entry, key, sname, len)) {
-      /* CRITICAL SECTION BEGIN */
+      /* CRITICAL CODE BEGIN */
       db = (entry->sym_ops == &dataBlockSym) ? entry->sym_value.db : NULL;
       entry->sym_ops = &intScalar; /* avoid clash in case of interrupts */
       Unref(db);
@@ -1163,7 +1171,7 @@ int h_insert(h_table_t *table, const char *sname, Symbol *sym)
 	entry->sym_value.db = Ref(db);
       } else entry->sym_value = sym->value;
       entry->sym_ops = sym->ops;   /* change ops only AFTER value updated */
-      /* CRITICAL SECTION END */
+      /* CRITICAL CODE END */
       return 1; /* old entry replaced */
     }
   }
@@ -1191,7 +1199,7 @@ int h_insert(h_table_t *table, const char *sname, Symbol *sym)
       h_error("insufficient memory to store new hash entry");
       return -1;
     }
-    /* CRITICAL SECTION BEGIN */
+    /* CRITICAL CODE BEGIN */
     table->slot = memset(new, 0, nbytes); /* empty new slots */
     table->size = new_size;
     table->mask = mask;
@@ -1206,7 +1214,7 @@ int h_insert(h_table_t *table, const char *sname, Symbol *sym)
       }
     }
     h_free(old);
-    /* CRITICAL SECTION END */
+    /* CRITICAL CODE END */
   }
 
   /* Create new entry. */
@@ -1222,11 +1230,11 @@ int h_insert(h_table_t *table, const char *sname, Symbol *sym)
 
   /* Insert new entry. */
   index = key & table->mask;
-  /* CRITICAL SECTION BEGIN */
+  /* CRITICAL CODE BEGIN */
   entry->next = table->slot[index];
   table->slot[index] = entry;
   ++table->number;
-  /* CRITICAL SECTION BEGIN */
+  /* CRITICAL CODE END */
   return 0; /* a new entry was created */
 }
 
