@@ -5,7 +5,7 @@
  *
  *-----------------------------------------------------------------------------
  *
- *	Copyright (C) 1999-2006 Eric Thibaut.
+ *	Copyright (C) 1999-2008 Eric Thibaut.
  *
  *	This file is part of Yeti.
  *
@@ -26,8 +26,26 @@
  *-----------------------------------------------------------------------------
  *
  * History:
- *	$Id: yeti_misc.c,v 1.6 2007/05/01 20:23:39 eric Exp $
+ *	$Id: yeti_misc.c,v 1.9 2008/02/14 11:14:59 eric Exp $
  *	$Log: yeti_misc.c,v $
+ *	Revision 1.9  2008/02/14 11:14:59  eric
+ *	- Fix functions: identof, is_scalar, is_vector, is_matrix,
+ *	  is_integer, is_real, is_complex, is_string, and
+ *	  is_numerical when argument is an L-value. Thanks to
+ *	  "sguieu" for reporting this bug on Yorick forum at
+ *	  SourceForge.
+ *	- Functions mem_copy and mem_copy also fixed to prevent
+ *	  this problem.
+ *
+ *	Revision 1.8  2007/12/26 10:59:28  eric
+ *	 - Update for management of dimension lists.
+ *	 - Fix encoding.
+ *	 - Minor fix in window_geometry.
+ *
+ *	Revision 1.7  2007/07/27 07:43:58  eric
+ *	 - Changes to globalize version number in the form:
+ *	   MAJOR.MINOR.MICRO[SUFFIX] -- i.e. SUFFIX is optional.
+ *
  *	Revision 1.6  2007/05/01 20:23:39  eric
  *	 - Fixed some compiler warnings.
  *
@@ -69,7 +87,7 @@
 # define USE_FASTER_DIVIDE_Z 0
 #elif (YORICK_VERSION_MAJOR == 1 && YORICK_VERSION_MINOR >= 6)
 # define USE_FASTER_DIVIDE_Z 0
-#elif (YORICK_VERSION_MAJOR == 1 && YORICK_VERSION_MINOR == 5 &&  YORICK_VERSION_MICRO >= 15)
+#elif (YORICK_VERSION_MAJOR == 1 && YORICK_VERSION_MINOR == 5 && YORICK_VERSION_MICRO >= 15)
 # define USE_FASTER_DIVIDE_Z 0
 #else
 # define USE_FASTER_DIVIDE_Z 1
@@ -142,6 +160,7 @@ static void append_to_default_path(const char *part1, const char *part2);
 static void set_path(int argc);
 #endif /* PLUG_IN */
 static void globalize_string(const char *name, const char *value);
+static void globalize_long(const char *name, long value);
 
 
 #if USE_FASTER_DIVIDE_Z
@@ -151,7 +170,9 @@ static void fast_DivideZ(Operand *l, Operand *r);
 void Y_yeti_init(int argc)
 {
   static int first_time = 1;
-  char buf[128];
+  const char *version = YETI_STRINGIFY(YETI_VERSION_MAJOR) "." \
+    YETI_STRINGIFY(YETI_VERSION_MINOR) "." \
+    YETI_STRINGIFY(YETI_VERSION_MICRO) YETI_VERSION_SUFFIX;
 
   if (first_time) {
     /* This is the first time _yeti_init has been called. */
@@ -178,7 +199,7 @@ void Y_yeti_init(int argc)
 	   execution from source directory and append launch directory
 	   to list of search paths. */
 	append_to_default_path(pathtmp, (char *)0);
-      
+
       } else {
 	/* Tail of launch directory does match "launch": assume standard
 	   installation. */
@@ -209,11 +230,13 @@ void Y_yeti_init(int argc)
 #ifndef PLUG_IN
   globalize_string("YETI_HOME", yeti_home);
 #endif /* not PLUG_IN */
-  sprintf(buf, "%d.%d.%d",
-	  YETI_VERSION_MAJOR, YETI_VERSION_MINOR, YETI_VERSION_MICRO);
-  globalize_string("YETI_VERSION", buf);
+  globalize_string("YETI_VERSION", version);
+  globalize_long("YETI_VERSION_MAJOR", YETI_VERSION_MAJOR);
+  globalize_long("YETI_VERSION_MINOR", YETI_VERSION_MINOR);
+  globalize_long("YETI_VERSION_MICRO", YETI_VERSION_MICRO);
+  globalize_string("YETI_VERSION_SUFFIX", YETI_VERSION_SUFFIX);
   if (! CalledAsSubroutine()) {
-    yeti_push_string_value(buf);
+    yeti_push_string_value(version);
   }
 }
 
@@ -272,7 +295,7 @@ static void append_to_default_path(const char *part1, const char *part2)
   }
   path[len] = 0;
   if (default_path) p_free(default_path);
-  default_path = path;  
+  default_path = path;
 }
 
 static void globalize_function(const char *name, BuiltIn *value, int quiet)
@@ -325,6 +348,16 @@ static void globalize_string(const char *name, const char *value)
   obj->value.q[0] = p_strcpy(value);
 }
 
+static void globalize_long(const char *name, long value)
+{
+  long index = Globalize(name, 0L);
+  DataBlock *old = (globTab[index].ops == &dataBlockSym ?
+		    globTab[index].value.db : 0);
+  globTab[index].ops = &longScalar; /* in case of interrupt */
+  globTab[index].value.l = value;
+  Unref(old);
+}
+
 #if USE_FASTER_DIVIDE_Z
 /* Faster code for complex division (save 1 division out of 3 with
    respect to original Yorick DivideZ code resulting in ~33% faster
@@ -422,7 +455,7 @@ void Y_mem_copy(int argc)
   Symbol *s;
 
   if (argc != 2) YError("mem_copy takes exactly 2 arguments");
-  address = get_address(sp-1);
+  address = get_address(sp - 1);
   s = (sp->ops == &referenceSym) ? &globTab[sp->index] : sp;
   if (s->ops == &doubleScalar) {
     (void)memcpy(address, &(s->value.d), sizeof(double));
@@ -441,44 +474,36 @@ void Y_mem_copy(int argc)
 
 void Y_mem_peek(int argc)
 {
-  Symbol *s, *stack = sp-argc+1;
+  Symbol *s, *stack = sp - argc + 1;
   StructDef *base;
   Array *array;
   void *address;
 
   if (argc < 2) YError("mem_peek takes at least 2 arguments");
   address = get_address(stack);
-  s = stack+1;
+  s = stack + 1;
   if (s->ops == &referenceSym) s = &globTab[s->index];
   if (s->ops != &dataBlockSym || s->value.db->ops != &structDefOps)
     YError("expected type definition as second argument");
   base = (StructDef *)s->value.db;
   if (base->dataOps->typeID < T_CHAR || base->dataOps->typeID > T_COMPLEX)
     YError("only basic data types are supported");
-  build_dimlist(stack+2, argc-2);
+  build_dimlist(stack + 2, argc - 2);
   array = PushDataBlock(NewArray(base, tmpDims));
   memcpy(array->value.c, address, array->type.number*array->type.base->size);
 }
 
 static void *get_address(Symbol *s)
 {
-  if (s->ops == &referenceSym) s = &globTab[s->index];
-#if 0 /* sizeof(int) != sizeof(size_t) on 64-bit machine */
-  if (s->ops == &intScalar) return (void *)s->value.i;
-#endif
-  if (s->ops == &longScalar) return (void *)s->value.l;
-  if (s->ops == &dataBlockSym) {
-    Array *array = (Array *)s->value.db;
-    if (array->ops->isArray && ! array->type.dims) {
-#if 0 /* sizeof(int) != sizeof(size_t) on 64-bit machine */
-      if (array->ops == &intOps) return (void *)array->value.i[0];
-#endif
-      if (array->ops == &longOps) return (void *)array->value.l[0];
-      if (array->ops == &pointerOps) return array->value.p[0];
-    }
+  Operand op;
+  if (! s->ops) YError("unexpected keyword argument");
+  s->ops->FormOperand(s, &op);
+  if (op.type.dims == (Dimension *)0) {
+    if (op.ops->typeID == T_LONG) return (void *)*(long *)op.value;
+    if (op.ops->typeID == T_POINTER) return (void *)*(void **)op.value;
   }
-  YError("bad address (expecting integer or pointer scalar)");
-  return 0; /* avoid compiler warning */
+  YError("bad address (expecting long integer or pointer scalar)");
+  return (void *)0; /* avoid compiler warning */
 }
 
 /* The following function is a pure copy of BuildDimList in 'ops3.c' of
@@ -604,17 +629,18 @@ void Y_window_geometry(int argc)
   GpTransform *transform;
   double one_pixel, dpi, xbias, ybias, width, height;
 
-  if (argc == 1) {
-    if (YNotNil(sp)) {
-      win = (int)YGetInteger(sp);
-      if (win < 0 || win >= GH_NDEVS) YError("bad graphic window number");
-    } else if ((win = GhGetPlotter()) < 0 || win >= GH_NDEVS) {
-      PushDataBlock(RefNC(&nilDB));
-      return;
-    }
-  } else {
+  if (argc != 1) {
     YError("window_geometry takes exactly one, possibly nil, argument");
   }
+  if (YNotNil(sp)) {
+    win = (int)YGetInteger(sp);
+  } else {
+    win = GhGetPlotter();
+  }
+  if (win < 0 || win >= GH_NDEVS || ! ghDevices[win].display) {
+    PushDataBlock(RefNC(&nilDB));
+    return;
+  }
 
   /* NDC -> pixel coordinate transform:
    *   XPIX = (int)(XSCALE*XNDC + XOFFSET)
@@ -661,7 +687,7 @@ void Y_window_geometry(int argc)
   }
 
   /* Build result array: [DPI, ONE_PIXEL, XBIAS, YBIAS, WIDTH, HEIGHT] */
-  geom = YETI_PUSH_NEW_D(ynew_dim(6, NULL));
+  geom = YETI_PUSH_NEW_D(yeti_start_dimlist(6));
   geom[0] = dpi;
   geom[1] = one_pixel;
   geom[2] = xbias;
@@ -751,7 +777,7 @@ static void alarm_callback(void *context)
 {
   Function *task;
   alarm_context_t *this = (alarm_context_t *)context;
-  
+
   if (! (task = this->task) && this->index >= 0) {
     Symbol *sym = &globTab[this->index];
     if (sym->ops == &dataBlockSym) {
@@ -820,7 +846,7 @@ void Y_set_alarm(int nargs)
   } else {
     YError("expecting function or function name");
   }
- 
+
   if (! alarm_free) {
     int i, n = 8;
     alarm_context_t *new = p_malloc(n*sizeof(alarm_context_t));
@@ -842,9 +868,9 @@ void Y_set_alarm(int nargs)
   *prev = this;
 
   if (task) this->task = Ref(task);
-  
+
   p_set_alarm(secs, alarm_callback, this);
-  PushDoubleValue(time); 
+  PushDoubleValue(time);
 }
 
 /*---------------------------------------------------------------------------*/
@@ -898,7 +924,7 @@ void Y_get_encoding(int argc)
   if (argc!=1) YError("get_encoding takes exactly one argument");
   name = YGetString(sp);
   if (name) {
-    long *result = YETI_PUSH_NEW_L(ynew_dim(32, NULL));
+    long *result = YETI_PUSH_NEW_L(yeti_start_dimlist(32));
     int i, c = name[0];
     for (i=0 ; i<ndb ; ++i) {
       if (c==db[i].name[0] && ! strcmp(name, db[i].name)) {
@@ -1009,26 +1035,6 @@ void Y_machine_constant(int argc)
 /*---------------------------------------------------------------------------*/
 /* SYMBOLS */
 
-void Y_identof(int argc)
-{
-  Symbol *s;
-  long typeID;
-  if (argc != 1) YError("identof takes exactly one argument");
-  s = YETI_DEREF_SYMBOL(sp);
-  if (s->ops==&doubleScalar) {
-    typeID = T_DOUBLE;
-  } else if (s->ops==&longScalar) {
-    typeID = T_LONG;
-  } else if (s->ops==&intScalar) {
-    typeID = T_INT;
-  } else if (s->ops==&dataBlockSym) {
-    typeID = s->value.db->ops->typeID;
-  } else {
-    YError("unexpected argument");
-  }
-  PushLongValue(typeID);
-}
-
 void Y_nrefsof(int argc)
 {
   Operand op;
@@ -1039,7 +1045,7 @@ void Y_nrefsof(int argc)
 
 void Y_symbol_exists(int argc)
 {
-  if (argc!=1) YError("symbol_exists takes exactly one argument");
+  if (argc != 1) YError("symbol_exists takes exactly one argument");
   PushIntValue(HashFind(&globalTable, YGetString(sp), 0L));
 }
 
@@ -1078,7 +1084,7 @@ void Y_symbol_names(int argc)
   }
   if (flags == -1) {
     /* Return names of all symbols ever defined. */
-    ret = YETI_PUSH_NEW_Q(ynew_dim(nitems, NULL));
+    ret = YETI_PUSH_NEW_Q(yeti_start_dimlist(nitems));
     for (i=0 ; i<nitems ; ++i) {
       ret[i] = p_strcpy(globalTable.names[i]);
     }
@@ -1129,7 +1135,7 @@ void Y_symbol_names(int argc)
 	PushDataBlock(RefNC(&nilDB));
 	return;
       }
-      ret = YETI_PUSH_NEW_Q(ynew_dim(number, NULL));
+      ret = YETI_PUSH_NEW_Q(yeti_start_dimlist(number));
     }
     for (i=0 ; i<nitems ; ++i) {
       if (globTab[i].ops != &dataBlockSym) {
@@ -1227,43 +1233,217 @@ void Y_insure_temporary(int argc)
 /*---------------------------------------------------------------------------*/
 /* OBJECTS PROPERTIES */
 
+
+#if 0
+/* The function get_info sets members of the structure pointed by TYPE and
+   returns NULL, or sets type->base to NULL and returns address of
+   DataBlock to non-Array object. */
+static DataBlock *get_info(Member *type, Symbol *s);
+
+
+/* A stack Symbol can have 4 different values: intScalar, longScalar,
+   doubleScalar, datablockSym or referenceSym.  For an array, the datablock
+   may be of type lvalue. */
+
+static DataBlock *get_info(Member *type, Symbol *s)
+{
+  DataBlock *db = (DataBlock *)0;
+  for (;;) {
+    if (s->ops == &doubleScalar) {
+      type->base = &doubleStruct;
+      type->dims = (Dimension *)0;
+      type->number = 1;
+      break;
+    } else if (s->ops == &longScalar) {
+      type->base = &longStruct;
+      type->dims = (Dimension *)0;
+      type->number = 1;
+      break;
+    } else if (s->ops == &intScalar) {
+      type->base = &intStruct;
+      type->dims = (Dimension *)0;
+      type->number = 1;
+      break;
+    } else if (s->ops == &dataBlockSym) {
+      db = s->value.db;
+      if (db->ops == &lvalueOps) {
+        LValue *lvalue = (LValue *)db;
+        type->base = lvalue->type.base;
+        type->dims = lvalue->type.dims;
+        type->number = lvalue->type.number;
+      } else if (db->ops->isArray) {
+        Array *array = (Array *)db;
+        type->base = array->type.base;
+        type->dims = array->type.dims;
+        type->number = array->type.number;
+      } else {
+        type->base = 0;
+        type->dims = (Dimension *)0;
+        type->number = 0;
+      }
+      break;
+    } else if (s->ops == &referenceSym) {
+      s = &globTab[s->index];
+    } else {
+      YError("unexpected keyword argument");
+    }
+  }
+  return (type->base ? 0 : db);
+}
+
+static int get_dims(Symbol *s, Dimension **dims);
+/* Gets the dimension list of stack symbol S, taking care of L-values and
+   of following references, and returns true for array and false
+   otherwise. */
+
+static int get_dims(Symbol *s, Dimension **dims)
+{
+  for (;;) {
+    if (s->ops == &dataBlockSym) {
+      DataBlock *db = s->value.db;
+      if (db->ops == &lvalueOps) {
+        LValue *lvalue = (LValue *)db;
+        *dims = lvalue->type.dims;
+	return 1; /* an L-value is always an array, see is_array */
+      } else if (db->ops->isArray) {
+        Array *array = (Array *)db;
+        *dims = array->type.dims;
+        return 1;
+      } else {
+        *dims = (Dimension *)0;
+        return 0;
+      }
+    } else if (s->ops == &referenceSym) {
+      s = &globTab[s->index];
+    } else if (s->ops != (OpTable *)0) {
+      /* Must be one of: intScalar, longScalar, or doubleScalar. */
+      *dims = (Dimension *)0;
+      return 1;
+    } else {
+      /* keyword */
+      *dims = (Dimension *)0;
+      return 0;
+    }
+  }
+}
+
+#endif
+
+static int get_type(Symbol *s);
+/* Returns the type identifier (typeID) of stack symbol S, taking care of
+   L-values and of following references.  Keyword symbols yield -1. */
+
+static int get_type(Symbol *s)
+{
+  for (;;) {
+    if (s->ops == &dataBlockSym) {
+      DataBlock *db = s->value.db;
+      if (db->ops == &lvalueOps) {
+	return ((LValue *)db)->type.base->dataOps->typeID;
+      } else {
+        return db->ops->typeID;
+      }
+    } else if (s->ops == &referenceSym) {
+      s = &globTab[s->index];
+    } else if (s->ops == &doubleScalar) {
+      return T_DOUBLE;
+    } else if (s->ops == &longScalar) {
+      return T_LONG;
+    } else if (s->ops == &intScalar) {
+      return T_INT;
+    } else {
+      /* Must be a keyword. */
+      return -1;
+    }
+  }
+}
+
+void Y_identof(int argc)
+{
+  long type;
+  if (argc != 1) YError("identof takes exactly one argument");
+  type = get_type(sp);
+  if (type == -1) YError("unexpected keyword argument");
+  PushLongValue(type);
+}
+
 void Y_is_scalar(int nargs)
 {
   Operand op;
   Symbol *s;
+  Operations *ops;
+  int result;
 
   if (nargs != 1) YError("is_scalar takes exactly one argument");
-  s = YETI_DEREF_SYMBOL(sp);
-  PushIntValue((s->ops == &intScalar || s->ops == &longScalar ||
-		s->ops == &doubleScalar) ||
-	       (s->ops == &dataBlockSym && s->value.db->ops->isArray &&
-		! s->ops->FormOperand(s, &op)->type.dims));
+  s = sp;
+  for (;;) {
+    if (s->ops == &dataBlockSym) {
+      ops =  s->value.db->ops;
+      result = ((ops->isArray || ops == &lvalueOps)
+		&& s->ops->FormOperand(s, &op)->type.dims == (Dimension *)0);
+      break;
+    } else if (s->ops != &referenceSym) {
+      /* Must be one of: intScalar, longScalar, or doubleScalar. */
+      result = 1;
+      break;
+    }
+    s = &globTab[s->index];
+  }
+  PushIntValue(result);
 }
 
 void Y_is_vector(int nargs)
 {
-  Dimension *dims;
   Operand op;
   Symbol *s;
+  Dimension *dims;
+  Operations *ops;
+  int result;
 
   if (nargs != 1) YError("is_vector takes exactly one argument");
-  s = YETI_DEREF_SYMBOL(sp);
-  PushIntValue((s->ops == &dataBlockSym && s->value.db->ops->isArray &&
-		(dims = s->ops->FormOperand(s, &op)->type.dims) &&
-		! dims->next));
+  s = sp;
+  for (;;) {
+    if (s->ops == &dataBlockSym) {
+      ops =  s->value.db->ops;
+      result = ((ops->isArray || ops == &lvalueOps)
+		&& (dims = s->ops->FormOperand(s, &op)->type.dims) != (Dimension *)0
+		&& dims->next == (Dimension *)0);
+      break;
+    } else if (s->ops != &referenceSym) {
+      /* Must be one of: intScalar, longScalar, or doubleScalar. */
+      result = 0;
+      break;
+    }
+    s = &globTab[s->index];
+  }
+  PushIntValue(result);
 }
 
 void Y_is_matrix(int nargs)
 {
-  Dimension *dims;
   Operand op;
   Symbol *s;
+  Dimension *dims;
+  Operations *ops;
+  int result;
 
   if (nargs != 1) YError("is_matrix takes exactly one argument");
-  s = YETI_DEREF_SYMBOL(sp);
-  PushIntValue((s->ops == &dataBlockSym && s->value.db->ops->isArray &&
-		(dims = s->ops->FormOperand(s, &op)->type.dims) &&
-		dims->next && ! dims->next->next));
+  s = sp;
+  for (;;) {
+    if (s->ops == &dataBlockSym) {
+      ops =  s->value.db->ops;
+      result = ((ops->isArray || ops == &lvalueOps)
+		&& (dims = s->ops->FormOperand(s, &op)->type.dims) != (Dimension *)0
+		&& dims->next != (Dimension *)0 && dims->next->next == (Dimension *)0);
+      break;
+    } else if (s->ops != &referenceSym) {
+      /* Must be one of: intScalar, longScalar, or doubleScalar. */
+      result = 0;
+      break;
+    }
+    s = &globTab[s->index];
+  }
+  PushIntValue(result);
 }
 
 #if YETI_MUST_PROVIDE_IS_LIST
@@ -1279,7 +1459,7 @@ void Y_is_list(int nargs)
     PushIntValue(s->value.db->ops == &listOps);
 #else
     PushIntValue(ops->typeID == T_OPAQUE && ! strcmp(ops->typeName, "list"));
-#endif  
+#endif
   } else {
     PushIntValue(0);
   }
@@ -1288,66 +1468,56 @@ void Y_is_list(int nargs)
 
 void Y_is_integer(int nargs)
 {
-  Symbol *s;
-  int typeID;
+  int type;
   if (nargs != 1) YError("is_integer takes exactly one argument");
-  s = YETI_DEREF_SYMBOL(sp);
-  PushIntValue(s->ops == &intScalar || s->ops == &longScalar ||
-	       (s->ops == &dataBlockSym &&
-		(typeID = s->value.db->ops->typeID) >= T_CHAR &&
-		typeID <= T_LONG));
+  type = get_type(sp);
+  PushIntValue(type >= T_CHAR && type <= T_LONG);
 }
 
 void Y_is_real(int nargs)
 {
-  Symbol *s;
-  int typeID;
+  int type;
   if (nargs != 1) YError("is_real takes exactly one argument");
-  s = YETI_DEREF_SYMBOL(sp);
-  PushIntValue(s->ops == &doubleScalar ||
-	       (s->ops == &dataBlockSym &&
-		(typeID = s->value.db->ops->typeID) >= T_FLOAT &&
-		typeID <= T_DOUBLE));
+  type = get_type(sp);
+  PushIntValue(type == T_DOUBLE || type == T_FLOAT);
 }
 
 void Y_is_complex(int nargs)
 {
-  Symbol *s;
   if (nargs != 1) YError("is_complex takes exactly one argument");
-  s = YETI_DEREF_SYMBOL(sp);
-  PushIntValue((s->ops == &dataBlockSym &&
-		s->value.db->ops->typeID == T_COMPLEX));
+  PushIntValue(get_type(sp) == T_COMPLEX);
 }
 
 void Y_is_string(int nargs)
 {
-  Symbol *s;
   if (nargs != 1) YError("is_string takes exactly one argument");
-  s = YETI_DEREF_SYMBOL(sp);
-  PushIntValue((s->ops == &dataBlockSym &&
-		s->value.db->ops->typeID == T_STRING));
+  PushIntValue(get_type(sp) == T_STRING);
 }
 
 void Y_is_pointer(int nargs)
 {
-  Symbol *s;
   if (nargs != 1) YError("is_pointer takes exactly one argument");
-  s = YETI_DEREF_SYMBOL(sp);
-  PushIntValue((s->ops == &dataBlockSym &&
-		s->value.db->ops->typeID == T_POINTER));
+  PushIntValue(get_type(sp) == T_POINTER);
 }
 
 void Y_is_numerical(int nargs)
 {
-  Symbol *s;
-  int typeID;
+  int result;
   if (nargs != 1) YError("is_numerical takes exactly one argument");
-  s = YETI_DEREF_SYMBOL(sp);
-  PushIntValue(s->ops == &intScalar || s->ops == &longScalar ||
-	       s->ops == &doubleScalar ||
-	       (s->ops == &dataBlockSym &&
-		(typeID = s->value.db->ops->typeID) >= T_CHAR &&
-		typeID <= T_COMPLEX));
+  switch (get_type(sp)) {
+  case T_CHAR:
+  case T_SHORT:
+  case T_INT:
+  case T_LONG:
+  case T_FLOAT:
+  case T_DOUBLE:
+  case T_COMPLEX:
+    result = 1;
+    break;
+  default:
+    result = 0;
+  }
+  PushIntValue(result);
 }
 
 /*---------------------------------------------------------------------------*/
@@ -1355,7 +1525,7 @@ void Y_is_numerical(int nargs)
 
 void Y_unref(int argc)
 {
-  if (argc!=1) YError("unref takes exactly one argument");
+  if (argc != 1) YError("unref takes exactly one argument");
   if (sp->ops == &referenceSym) {
     /* Replace reference without augmenting the number of references of
        the data block object if it is an array. */
@@ -1422,7 +1592,7 @@ void Y_get_includes(int argc)
     YError("get_includes takes exactly one nil argument");
   if (sourceTab.nItems > 0) {
     int i;
-    char **s = YETI_PUSH_NEW_Q(ynew_dim(sourceTab.nItems, NULL));
+    char **s = YETI_PUSH_NEW_Q(yeti_start_dimlist(sourceTab.nItems));
     for (i=0 ; i<sourceTab.nItems ; ++i) s[i] = p_strcpy(sourceTab.names[i]);
   } else {
     PushDataBlock(RefNC(&nilDB));
@@ -1642,4 +1812,11 @@ static void smooth_single(double *x, double p25, double p50, double p75,
   }
 }
 
-/*---------------------------------------------------------------------------*/
+/*---------------------------------------------------------------------------*
+ * Local Variables:                                                          *
+ * mode: C                                                                   *
+ * tab-width: 8                                                              *
+ * fill-column: 75                                                           *
+ * coding: latin-1                                                           *
+ * End:                                                                      *
+ *---------------------------------------------------------------------------*/
