@@ -27,7 +27,7 @@
  *-----------------------------------------------------------------------------
  *
  * History:
- *	$Id: yeti_new.c,v 1.2 2007/04/19 17:05:20 eric Exp $
+ *	$Id: yeti_new.c,v 1.2 2007/04/19 17:05:20 eric Exp eric $
  *	$Log: yeti_new.c,v $
  *	Revision 1.2  2007/04/19 17:05:20  eric
  *	Bug in make_dimlist fixed.
@@ -232,3 +232,12 @@ static long CHECK_DIMS(const void *array, const long dims[])
 #undef type_t
 #undef CHECK_DIMS
 #endif /* _YETI_NEW_C defined */
+
+/*---------------------------------------------------------------------------*
+ * Local Variables:                                                          *
+ * mode: C                                                                   *
+ * tab-width: 8                                                              *
+ * fill-column: 75                                                           *
+ * coding: latin-1                                                           *
+ * End:                                                                      *
+ *---------------------------------------------------------------------------*/
