@@ -5,7 +5,7 @@
  *
  *-----------------------------------------------------------------------------
  *
- *	Copyright (C) 2005-2006 Eric Thibaut.
+ *	Copyright (C) 2005-2007 Eric Thibaut.
  *
  *	This file is part of Yeti.
  *
@@ -26,11 +26,18 @@
  *-----------------------------------------------------------------------------
  *
  * History:
- *	$Id: yeti_sparse.c,v 1.1 2006/07/19 14:53:11 eric Exp $
+ *	$Id: yeti_sparse.c,v 1.3 2007/03/28 12:08:03 eric Exp $
  *	$Log: yeti_sparse.c,v $
+ *	Revision 1.3  2007/03/28 12:08:03  eric
+ *	 - A few minor fixes to avoid compiler warnings.
+ *
+ *	Revision 1.2  2007/03/23 16:28:34  eric
+ *	 - Restricted possible values for JOB/FLAGS in evaluation
+ *	   of matrix products: must be 0 (default) for direct
+ *	   product and 1 for transpose product.
+ *
  *	Revision 1.1  2006/07/19 14:53:11  eric
  *	Initial revision
- *
  */
 
 #include <stdlib.h>
@@ -39,9 +46,19 @@
 #include <ydata.h>
 #include <yio.h>
 
+/* Debug level: 0 or undefined = none,
+ *              1 = perform assertions,
+ *              2 = verbose debug.
+ */
+#undef YETI_SPARSE_DEBUG
+
 extern BuiltIn Y_sparse_matrix, Y_is_sparse_matrix;
 extern BuiltIn Y_mvmult;
 
+#if ! (defined(__GNUC__) && __GNUC__ > 1)
+extern void YError(const char *msg) __attribute__ ((noreturn));
+#endif
+
 /*--------------------------------------------------------------------------*/
 /* IMPLEMENTATION OF SPARSE MATRICES AS OPAQUE YORICK OBJECTS */
 
@@ -142,7 +159,7 @@ static void pop_to(Symbol *owner, int cleanup);
 void Y_sparse_matrix(int argc)
 {
   size_t off1, off2, nint, size;
-  size_t i, number, ndims1, nelem1, len1, ndims2, nelem2, len2;
+  size_t i, number=0, ndims1, nelem1, len1, ndims2, nelem2, len2;
   long *dims1, *idx1, *dims2, *idx2;
   size_t *row_indices, *col_indices;
   double *coefs, *nonzero;
@@ -305,14 +322,17 @@ static long *get_dimlist(Symbol *s, size_t *ndims_ptr, size_t *nelem_ptr)
     } else {
       goto bad_dimlist;
     }
-    i = ndims;
     nelem = 1;
     for (i=0 ; i<ndims ; ++i) {
       if (dimlist[i] <= 0) goto bad_dimlist;
       nelem *= dimlist[i];
     }
-    if (ndims_ptr) *ndims_ptr = ndims;
-    if (nelem_ptr) *nelem_ptr = nelem;
+    if (ndims_ptr) {
+      *ndims_ptr = ndims;
+    }
+    if (nelem_ptr) {
+      *nelem_ptr = nelem;
+    }
     return dimlist;
   }
 
@@ -359,10 +379,14 @@ static void sparse_eval(Operand *op0)
   }
 
   /* Get the 'matrix'. */
+#if defined(YETI_SPARSE_DEBUG) && YETI_SPARSE_DEBUG >= 1
   sym = (stack->ops == &referenceSym) ? &globTab[stack->index] : stack;
   if (sym->ops != &dataBlockSym || sym->value.db->ops != &sparseOps)
     YError("unexpected non-sparse matrix object (must be a BUG!)");
   sparse = (sparse_t *)sym->value.db;
+#else
+  sparse = (sparse_t *)stack->value.db;
+#endif
 
   /* Get the flags. */
   if (sp - stack == 2) {
@@ -370,12 +394,16 @@ static void sparse_eval(Operand *op0)
   } else {
     flags = 0;
   }
-  if (flags) {
+  if (! flags) {
+    inp = &sparse->col;
+    out = &sparse->row;
+  } else if (flags == 1) {
     inp = &sparse->row;
     out = &sparse->col;
   } else {
-    inp = &sparse->col;
-    out = &sparse->row;
+    inp = 0; /* avoids compiler warning */
+    out = 0; /* avoids compiler warning */
+    YError("unsupported job value (should be 0 or 1)");
   }
 
   /* Get the input 'vector'. */
@@ -564,6 +592,9 @@ void Y_mvmult(int argc)
   } else {
     /* Get the optional flags. */
     flags = (argc == 3 ? get_flags(sp, 0) : 0);
+    if ((unsigned int)flags > 1U) {
+      YError("unsupported job value (should be 0 or 1)");
+    }
 
     /* Get the 'matrix' A. */
     switch (op.ops->typeID) {
@@ -680,4 +711,11 @@ static size_t pack_dimlist(const Dimension *dims, size_t dimlist[],
   return n;
 }
 
-/*---------------------------------------------------------------------------*/
+/*---------------------------------------------------------------------------*
+ * Local Variables:                                                          *
+ * mode: C                                                                   *
+ * tab-width: 8                                                              *
+ * fill-column: 75                                                           *
+ * coding: latin-1                                                           *
+ * End:                                                                      *
+ *---------------------------------------------------------------------------*/
